package sun.awt.image;

import java.awt.Rectangle;
import java.awt.GraphicsConfiguration;
import sun.java2d.loops.CompositeType;
import java.awt.image.DataBuffer;
import sun.java2d.SunGraphics2D;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.awt.image.ColorModel;
import java.awt.image.DirectColorModel;
import java.awt.image.IndexColorModel;
import sun.java2d.loops.SurfaceType;
import sun.java2d.loops.RenderLoops;
import java.awt.image.BufferedImage;
import sun.java2d.SurfaceData;

public class BufImgSurfaceData extends SurfaceData
{
    BufferedImage bufImg;
    private BufferedImageGraphicsConfig graphicsConfig;
    RenderLoops solidloops;
    private static final int DCM_RGBX_RED_MASK = -16777216;
    private static final int DCM_RGBX_GREEN_MASK = 16711680;
    private static final int DCM_RGBX_BLUE_MASK = 65280;
    private static final int DCM_555X_RED_MASK = 63488;
    private static final int DCM_555X_GREEN_MASK = 1984;
    private static final int DCM_555X_BLUE_MASK = 62;
    private static final int DCM_4444_RED_MASK = 3840;
    private static final int DCM_4444_GREEN_MASK = 240;
    private static final int DCM_4444_BLUE_MASK = 15;
    private static final int DCM_4444_ALPHA_MASK = 61440;
    private static final int DCM_ARGBBM_ALPHA_MASK = 16777216;
    private static final int DCM_ARGBBM_RED_MASK = 16711680;
    private static final int DCM_ARGBBM_GREEN_MASK = 65280;
    private static final int DCM_ARGBBM_BLUE_MASK = 255;
    private static final int CACHE_SIZE = 5;
    private static RenderLoops[] loopcache;
    private static SurfaceType[] typecache;
    
    private static native void initIDs(Class p0, Class p1);
    
    public static SurfaceData createData(BufferedImage bufferedImage) {
        if (bufferedImage == null) {
            throw new NullPointerException("BufferedImage cannot be null");
        }
        ColorModel colorModel = bufferedImage.getColorModel();
        SurfaceData surfaceData;
        switch (bufferedImage.getType()) {
            case 4: {
                surfaceData = createDataIC(bufferedImage, SurfaceType.IntBgr);
                break;
            }
            case 1: {
                surfaceData = createDataIC(bufferedImage, SurfaceType.IntRgb);
                break;
            }
            case 2: {
                surfaceData = createDataIC(bufferedImage, SurfaceType.IntArgb);
                break;
            }
            case 3: {
                surfaceData = createDataIC(bufferedImage, SurfaceType.IntArgbPre);
                break;
            }
            case 5: {
                surfaceData = createDataBC(bufferedImage, SurfaceType.ThreeByteBgr, 2);
                break;
            }
            case 6: {
                surfaceData = createDataBC(bufferedImage, SurfaceType.FourByteAbgr, 3);
                break;
            }
            case 7: {
                surfaceData = createDataBC(bufferedImage, SurfaceType.FourByteAbgrPre, 3);
                break;
            }
            case 8: {
                surfaceData = createDataSC(bufferedImage, SurfaceType.Ushort565Rgb, null);
                break;
            }
            case 9: {
                surfaceData = createDataSC(bufferedImage, SurfaceType.Ushort555Rgb, null);
                break;
            }
            case 13: {
                SurfaceType surfaceType;
                switch (colorModel.getTransparency()) {
                    case 1: {
                        if (SurfaceData.isOpaqueGray((IndexColorModel)colorModel)) {
                            surfaceType = SurfaceType.Index8Gray;
                            break;
                        }
                        surfaceType = SurfaceType.ByteIndexedOpaque;
                        break;
                    }
                    case 2: {
                        surfaceType = SurfaceType.ByteIndexedBm;
                        break;
                    }
                    case 3: {
                        surfaceType = SurfaceType.ByteIndexed;
                        break;
                    }
                    default: {
                        throw new InternalError("Unrecognized transparency");
                    }
                }
                surfaceData = createDataBC(bufferedImage, surfaceType, 0);
                break;
            }
            case 10: {
                surfaceData = createDataBC(bufferedImage, SurfaceType.ByteGray, 0);
                break;
            }
            case 11: {
                surfaceData = createDataSC(bufferedImage, SurfaceType.UshortGray, null);
                break;
            }
            case 12: {
                SurfaceType surfaceType2;
                switch (bufferedImage.getRaster().getSampleModel().getSampleSize(0)) {
                    case 1: {
                        surfaceType2 = SurfaceType.ByteBinary1Bit;
                        break;
                    }
                    case 2: {
                        surfaceType2 = SurfaceType.ByteBinary2Bit;
                        break;
                    }
                    case 4: {
                        surfaceType2 = SurfaceType.ByteBinary4Bit;
                        break;
                    }
                    default: {
                        throw new InternalError("Unrecognized pixel size");
                    }
                }
                surfaceData = createDataBP(bufferedImage, surfaceType2);
                break;
            }
            default: {
                WritableRaster raster = bufferedImage.getRaster();
                int numBands = raster.getNumBands();
                if (raster instanceof IntegerComponentRaster && raster.getNumDataElements() == 1 && ((IntegerComponentRaster)raster).getPixelStride() == 1) {
                    SurfaceType surfaceType4 = SurfaceType.AnyInt;
                    if ((colorModel instanceof DirectColorModel)) {
                        DirectColorModel directColorModel2 = (DirectColorModel)colorModel;
                        int alphaMask2 = directColorModel2.getAlphaMask();
                        int redMask2 = directColorModel2.getRedMask();
                        int greenMask2 = directColorModel2.getGreenMask();
                        int blueMask2 = directColorModel2.getBlueMask();
                        if (numBands == 3 && alphaMask2 == 0 && redMask2 == -16777216 && greenMask2 == 16711680 && blueMask2 == 65280) {
                            surfaceType4 = SurfaceType.IntRgbx;
                        }
                        else if (numBands == 4 && alphaMask2 == 16777216 && redMask2 == 16711680 && greenMask2 == 65280 && blueMask2 == 255) {
                            surfaceType4 = SurfaceType.IntArgbBm;
                        }
                        else {
                            surfaceType4 = SurfaceType.AnyDcm;
                        }
                    }
                    surfaceData = createDataIC(bufferedImage, surfaceType4);
                    break;
                }
                else if (raster instanceof ShortComponentRaster && raster.getNumDataElements() == 1 && ((ShortComponentRaster)raster).getPixelStride() == 1) {
                    SurfaceType surfaceType3 = SurfaceType.AnyShort;
                    IndexColorModel indexColorModel = null;
                    if ((colorModel instanceof DirectColorModel)) {
                        DirectColorModel directColorModel = (DirectColorModel)colorModel;
                        int alphaMask = directColorModel.getAlphaMask();
                        int redMask = directColorModel.getRedMask();
                        int greenMask = directColorModel.getGreenMask();
                        int blueMask = directColorModel.getBlueMask();
                        if (numBands == 3 && alphaMask == 0 && redMask == 63488 && greenMask == 1984 && blueMask == 62) {
                            surfaceType3 = SurfaceType.Ushort555Rgbx;
                        }
                        else if (numBands == 4 && alphaMask == 61440 && redMask == 3840 && greenMask == 240 && blueMask == 15) {
                            surfaceType3 = SurfaceType.Ushort4444Argb;
                        }
                    }
                    else if ((colorModel instanceof IndexColorModel)) {
                        indexColorModel = (IndexColorModel)colorModel;
                        if (indexColorModel.getPixelSize() == 12) {
                            if (SurfaceData.isOpaqueGray(indexColorModel)) {
                                surfaceType3 = SurfaceType.Index12Gray;
                            }
                            else {
                                surfaceType3 = SurfaceType.UshortIndexed;
                            }
                        }
                        else {
                            indexColorModel = null;
                        }
                    }
                    surfaceData = createDataSC(bufferedImage, surfaceType3, indexColorModel);
                    break;
                }
                else {
                    surfaceData = new BufImgSurfaceData(raster.getDataBuffer(), bufferedImage, SurfaceType.Custom);
                    break;
                }
            }
        }
        ((BufImgSurfaceData)surfaceData).initSolidLoops();
        return surfaceData;
    }
    
    public static SurfaceData createData(Raster raster, ColorModel colorModel) {
        throw new InternalError("SurfaceData not implemented for Raster/CM");
    }
    
    public static SurfaceData createDataIC(BufferedImage bufferedImage, SurfaceType surfaceType) {
        IntegerComponentRaster integerComponentRaster = (IntegerComponentRaster)bufferedImage.getRaster();
        BufImgSurfaceData bufImgSurfaceData = new BufImgSurfaceData(integerComponentRaster.getDataBuffer(), bufferedImage, surfaceType);
        bufImgSurfaceData.initRaster(integerComponentRaster.getDataStorage(), integerComponentRaster.getDataOffset(0) * 4, 0, integerComponentRaster.getWidth(), integerComponentRaster.getHeight(), integerComponentRaster.getPixelStride() * 4, integerComponentRaster.getScanlineStride() * 4, null);
        return bufImgSurfaceData;
    }
    
    public static SurfaceData createDataSC(BufferedImage bufferedImage, SurfaceType surfaceType, IndexColorModel indexColorModel) {
        ShortComponentRaster shortComponentRaster = (ShortComponentRaster)bufferedImage.getRaster();
        BufImgSurfaceData bufImgSurfaceData = new BufImgSurfaceData(shortComponentRaster.getDataBuffer(), bufferedImage, surfaceType);
        bufImgSurfaceData.initRaster(shortComponentRaster.getDataStorage(), shortComponentRaster.getDataOffset(0) * 2, 0, shortComponentRaster.getWidth(), shortComponentRaster.getHeight(), shortComponentRaster.getPixelStride() * 2, shortComponentRaster.getScanlineStride() * 2, indexColorModel);
        return bufImgSurfaceData;
    }
    
    public static SurfaceData createDataBC(BufferedImage bufferedImage, SurfaceType surfaceType, int n) {
        ByteComponentRaster byteComponentRaster = (ByteComponentRaster)bufferedImage.getRaster();
        BufImgSurfaceData bufImgSurfaceData = new BufImgSurfaceData(byteComponentRaster.getDataBuffer(), bufferedImage, surfaceType);
        ColorModel colorModel = bufferedImage.getColorModel();
        bufImgSurfaceData.initRaster(byteComponentRaster.getDataStorage(), byteComponentRaster.getDataOffset(n), 0, byteComponentRaster.getWidth(), byteComponentRaster.getHeight(), byteComponentRaster.getPixelStride(), byteComponentRaster.getScanlineStride(), (colorModel instanceof IndexColorModel) ? ((IndexColorModel)colorModel) : null);
        return bufImgSurfaceData;
    }
    
    public static SurfaceData createDataBP(BufferedImage bufferedImage, SurfaceType surfaceType) {
        BytePackedRaster bytePackedRaster = (BytePackedRaster)bufferedImage.getRaster();
        BufImgSurfaceData bufImgSurfaceData = new BufImgSurfaceData(bytePackedRaster.getDataBuffer(), bufferedImage, surfaceType);
        ColorModel colorModel = bufferedImage.getColorModel();
        bufImgSurfaceData.initRaster(bytePackedRaster.getDataStorage(), bytePackedRaster.getDataBitOffset() / 8, bytePackedRaster.getDataBitOffset() & 0x7, bytePackedRaster.getWidth(), bytePackedRaster.getHeight(), 0, bytePackedRaster.getScanlineStride(), (colorModel instanceof IndexColorModel) ? ((IndexColorModel)colorModel) : null);
        return bufImgSurfaceData;
    }
    
    @Override
    public RenderLoops getRenderLoops(SunGraphics2D sg2d) {
        if (sg2d.paintState <= 1 && sg2d.compositeState <= 0) {
            return this.solidloops;
        }
        return super.getRenderLoops(sg2d);
    }
    
    @Override
    public Raster getRaster(int n, int n2, int n3, int n4) {
        return this.bufImg.getRaster();
    }
    
    protected native void initRaster(Object p0, int p1, int p2, int p3, int p4, int p5, int p6, IndexColorModel p7);
    
    public BufImgSurfaceData(DataBuffer dataBuffer, BufferedImage bufImg, SurfaceType surfaceType) {
        super(SunWritableRaster.stealTrackable(dataBuffer), surfaceType, bufImg.getColorModel());
        this.bufImg = bufImg;
    }
    
    protected BufImgSurfaceData(SurfaceType surfaceType, ColorModel cm) {
        super(surfaceType, cm);
    }
    
    public void initSolidLoops() {
        this.solidloops = getSolidLoops(this.getSurfaceType());
    }
    
    public static synchronized RenderLoops getSolidLoops(SurfaceType dst) {
        int i = 4;
        while (i >= 0) {
            SurfaceType surfaceType = BufImgSurfaceData.typecache[i];
            if (surfaceType == dst) {
                return BufImgSurfaceData.loopcache[i];
            }
            if (surfaceType == null) {
                break;
            }
            --i;
        }
        RenderLoops renderLoops = SurfaceData.makeRenderLoops(SurfaceType.OpaqueColor, CompositeType.SrcNoEa, dst);
        System.arraycopy(BufImgSurfaceData.loopcache, 1, BufImgSurfaceData.loopcache, 0, 4);
        System.arraycopy(BufImgSurfaceData.typecache, 1, BufImgSurfaceData.typecache, 0, 4);
        BufImgSurfaceData.loopcache[4] = renderLoops;
        BufImgSurfaceData.typecache[4] = dst;
        return renderLoops;
    }
    
    @Override
    public SurfaceData getReplacement() {
        return SurfaceData.restoreContents(this.bufImg);
    }
    
    @Override
    public synchronized GraphicsConfiguration getDeviceConfiguration() {
        if (this.graphicsConfig == null) {
            this.graphicsConfig = BufferedImageGraphicsConfig.getConfig(this.bufImg);
        }
        return this.graphicsConfig;
    }
    
    @Override
    public Rectangle getBounds() {
        return new Rectangle(this.bufImg.getWidth(), this.bufImg.getHeight());
    }
    
    @Override
    protected void checkCustomComposite() {
    }
    
    @Override
    public Object getDestination() {
        return this.bufImg;
    }
    
    static {
        initIDs(IndexColorModel.class, BufImgSurfaceData.ICMColorData.class);
        BufImgSurfaceData.loopcache = new RenderLoops[5];
        BufImgSurfaceData.typecache = new SurfaceType[5];
    }
    
    public static final class ICMColorData
    {
        private long pData;
        
        private ICMColorData(long pData) {
            this.pData = 0L;
            this.pData = pData;
        }
    }
}
